/*
 * WindowListener.java
 *
 * Copyright (C) 2008-2009 Alessio Stalla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this library, you may extend
 * this exception to your version of the library, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 */

package snow.swing;

import java.awt.event.WindowEvent;

import org.armedbear.lisp.JavaObject;
import org.armedbear.lisp.LispObject;

public class WindowListener implements java.awt.event.WindowListener {

    private LispObject windowActivated;
    private LispObject windowClosed;
    private LispObject windowClosing;
    private LispObject windowDeactivated;
    private LispObject windowDeiconified;
    private LispObject windowIconified;
    private LispObject windowOpened;
	
    public WindowListener(LispObject windowActivated, LispObject windowClosed,
			  LispObject windowClosing, LispObject windowDeactivated,
			  LispObject windowDeiconified, LispObject windowIconified,
			  LispObject windowOpened) {
	super();
	this.windowActivated = windowActivated;
	this.windowClosed = windowClosed;
	this.windowClosing = windowClosing;
	this.windowDeactivated = windowDeactivated;
	this.windowDeiconified = windowDeiconified;
	this.windowIconified = windowIconified;
	this.windowOpened = windowOpened;
    }
	
    private static final void invokeDelegate(LispObject delegate, WindowEvent e) {
	if(delegate != null) {
	    delegate.execute(new JavaObject(e));
	}
    }
	
    @Override
    public void windowActivated(WindowEvent e) {
	invokeDelegate(windowActivated, e);
    }
    
    @Override
    public void windowClosed(WindowEvent e) {
	invokeDelegate(windowClosed, e);
    }

    @Override
    public void windowClosing(WindowEvent e) {
	invokeDelegate(windowClosing, e);
    }

    @Override
    public void windowDeactivated(WindowEvent e) {
	invokeDelegate(windowDeactivated, e);
    }

    @Override
    public void windowDeiconified(WindowEvent e) {
	invokeDelegate(windowDeiconified, e);
    }
    
    @Override
    public void windowIconified(WindowEvent e) {
	invokeDelegate(windowIconified, e);
    }
    
    @Override
    public void windowOpened(WindowEvent e) {
	invokeDelegate(windowOpened, e);
    }

}
