/*
 * ConsTreeModel.java
 *
 * Copyright (C) 2008-2009 Alessio Stalla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this library, you may extend
 * this exception to your version of the library, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 */

package snow.tree;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.armedbear.lisp.Cons;
import org.armedbear.lisp.Fixnum;
import org.armedbear.lisp.Lisp;
import org.armedbear.lisp.LispObject;
import org.armedbear.lisp.Symbol;

public class ConsTreeModel implements TreeModel {

	private Cons cons;
	private List<TreeModelListener> listeners = new ArrayList<TreeModelListener>();
	
	public ConsTreeModel(Cons cons) {
		super();
		this.cons = cons;
	}

    @Override
    public Object getChild(Object parent, int index) {
	if(parent instanceof Cons) {
	    return Symbol.NTH.execute(Fixnum.getInstance(index + 1), (Cons) parent);
	    
	} else {
	    return null;
	}
    }

    @Override
    public int getChildCount(Object parent) {
	if(parent instanceof Cons) {
	    return ((Fixnum) Symbol.LENGTH.execute((Cons) parent)).value - 1;
	} else {
	    return 0;
	}
    }

    @Override
    public int getIndexOfChild(Object parent, Object child) {
	if(parent == null || child == null) {
	    return -1;
	}
	if(Symbol.MEMBER.execute((LispObject) parent, cons) != Lisp.NIL) {
	    Object pos = Symbol.POSITION.execute((LispObject) child, (LispObject) parent);
	    if(pos instanceof Fixnum) {
		return ((Fixnum) pos).value - 1;
	    } else {
		return -1;
	    }
	} else {
	    return -1;
	}
    }

    @Override
    public Object getRoot() {
	return cons;
    }

    @Override
    public boolean isLeaf(Object node) {
	return Symbol.ATOM.execute((LispObject) node) != Lisp.NIL;
    }

    @Override
    public void addTreeModelListener(TreeModelListener l) {
	listeners.add(l);
    }
    
    @Override
    public void removeTreeModelListener(TreeModelListener l) {
	listeners.remove(l);	
    }
    
    @Override
    public void valueForPathChanged(TreePath path, Object newValue) {
	// TODO Auto-generated method stub
	
    }

}
